/*
**    TP 5ETI CPE Lyon
**    Copyright (C) 2012 Damien Rohmer
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**   This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/


#include <fstream>
#include <sstream>
#include <map>
#include <list>

#include <cmath>
#include <cstdlib>
#include <assert.h>

#include "animation.hpp"

namespace cpe
{
  const v3& animation::position(const unsigned int& frame, unsigned int bone) const
  {
    //check indices
    if(frame * bones>=v_positions.size())
    {
      std::ostringstream msg;
      msg<<"Error index "<<frame<<" outside bounds of size:"<<v_positions.size() / bones;
      throw cpe::exception_animation(msg.str(),__FILE__,__PRETTY_FUNCTION__,__LINE__);
    }
    if(bone>=bones)
    {
      std::ostringstream msg;
      msg<<"Error index "<<bone<<" outside bounds of size: "<< bones;
      throw cpe::exception_animation(msg.str(),__FILE__,__PRETTY_FUNCTION__,__LINE__);
    }
    //return index
    return v_positions[frame * bones + bone];
  }
  v3& animation::position(const unsigned int& frame, unsigned int bone)
  {
    //check indices
    if(frame * bones>=v_positions.size())
    {
      std::ostringstream msg;
      msg<<"Error index "<<frame<<" outside bounds of size:"<<v_positions.size() / bones;
      throw cpe::exception_animation(msg.str(),__FILE__,__PRETTY_FUNCTION__,__LINE__);
    }
    if(bone>=bones)
    {
      std::ostringstream msg;
      msg<<"Error index "<<bone<<" outside bounds of size: "<< bones;
      throw cpe::exception_animation(msg.str(),__FILE__,__PRETTY_FUNCTION__,__LINE__);
    }
    //return index
    return v_positions[frame * bones + bone];
  }
  const std::vector<v3>& animation::get_positions() const{return v_positions;}
  std::vector<v3>& animation::get_positions() {return v_positions;}

  const quaternion& animation::orientation(const unsigned int& frame, unsigned int bone) const
  {
    //check indices
    if(frame * bones>=v_orientations.size())
    {
      std::ostringstream msg;
      msg<<"Error index "<<frame<<" outside bounds of size:"<<v_orientations.size() / bones;
      throw cpe::exception_animation(msg.str(),__FILE__,__PRETTY_FUNCTION__,__LINE__);
    }
    if(bone>=bones)
    {
      std::ostringstream msg;
      msg<<"Error index "<<bone<<" outside bounds of size: "<< bones;
      throw cpe::exception_animation(msg.str(),__FILE__,__PRETTY_FUNCTION__,__LINE__);
    }
    //return index
    return v_orientations[frame * bones + bone];
  }
  quaternion& animation::orientation(const unsigned int& frame, unsigned int bone)
  {
    //check indices
    if(frame * bones>=v_orientations.size())
    {
      std::ostringstream msg;
      msg<<"Error index "<<frame<<" outside bounds of size:"<<v_orientations.size() / bones;
      throw cpe::exception_animation(msg.str(),__FILE__,__PRETTY_FUNCTION__,__LINE__);
    }
    if(bone>=bones)
    {
      std::ostringstream msg;
      msg<<"Error index "<<bone<<" outside bounds of size: "<< bones;
      throw cpe::exception_animation(msg.str(),__FILE__,__PRETTY_FUNCTION__,__LINE__);
    }
    //return index
    return v_orientations[frame * bones + bone];
  }
  const std::vector<quaternion>& animation::get_orientations() const{return v_orientations;}
  std::vector<quaternion>& animation::get_orientations() {return v_orientations;}

  unsigned int animation::number_of_bones() const{return bones;}
  unsigned int animation::number_of_frames() const{return v_positions.size()/bones;}

  animation::animation(unsigned int nb_bones, const std::string &filename) : bones(nb_bones)
  {
    std::ifstream fid(filename.c_str());
    if(!fid.good())
      throw exception_animation("Cannot open file "+filename,__FILE__,__FUNCTION__,__LINE__);

    std::string buffer;

    //read the whole file
    while(fid.good()==true)
    {
      //read line
      std::getline(fid,buffer);

      if(buffer.size()>0&&buffer[0] != '#')
      {
        v3 position;
        quaternion orientation;
        std::stringstream tokens_buffer(buffer);

        tokens_buffer>>position.x();
        tokens_buffer>>position.y();
        tokens_buffer>>position.z();
        tokens_buffer>>orientation.x();
        tokens_buffer>>orientation.y();
        tokens_buffer>>orientation.z();
        tokens_buffer>>orientation.w();

        v_positions.push_back(position);
        v_orientations.push_back(orientation);
      }
    }
  }
}


