/*
**    TP 5ETI CPE Lyon
**    Copyright (C) 2012 David Odin
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**   This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/



#ifndef ANIMATION_HPP
#define ANIMATION_HPP

#include <vector>

#include <v3.hpp>
#include <quaternion.hpp>
#include <exception_cpe.hpp>

namespace cpe
{

  /** \brief Container class for a animation. */
  class animation
  {
    /** \brief internal positions storage */
    std::vector<v3> v_positions;
    /** \brief internal orientation storage */
    std::vector<quaternion> v_orientations;

    unsigned bones;
    public:
    // ********************************************* //
    // ********************************************* //
    //  Access data
    // ********************************************* //
    // ********************************************* //

    /** \brief Accessor to the position value */
    const v3& position(const unsigned int& frame, unsigned bone) const;
    /** \brief Accessor to the position value */
    v3& position(const unsigned int& frame, unsigned bone);
    /** \brief return internal positions vector */
    const std::vector<v3>& get_positions() const;
    /** \brief return internal positions vector */
    std::vector<v3>& get_positions();

    /** \brief Accessor to the orientation value */
    const quaternion& orientation(const unsigned int& frame, unsigned bone) const;
    /** \brief Accessor to the orientation value */
    quaternion& orientation(const unsigned int& frame, unsigned bone);
    /** \brief return internal orientations vector */
    const std::vector<quaternion>& get_orientations() const;
    /** \brief return internal orientations vector */
    std::vector<quaternion>& get_orientations();

    // ********************************************* //
    // ********************************************* //
    //  Size
    // ********************************************* //
    // ********************************************* //

    /** \brief get the number of frames */
    unsigned int number_of_frames() const;

    /** \brief get the number of bones */
    unsigned int number_of_bones() const;

    /** \brief construct with a animation file */
    animation(unsigned int bones, const std::string &filename);

  };

  /** \brief Special exception class for animation */
  class exception_animation : public exception_cpe
  {
    public:

      /** \brief empty constructor */
      exception_animation():exception_cpe(){}
      /** \brief constructor with error information */
      exception_animation(const std::string& msg,const std::string& file,const std::string& caller,const int& line):exception_cpe(msg,file,caller,line){}
  };
}

#endif
