
/** TP 5ETI - CPE Lyon - 2012/2013 */


#ifndef SCENE_HPP
#define SCENE_HPP

#include <vector>

#include <matrix3.hpp>
#include <v3.hpp>
#include <quaternion.hpp>
#include <mesh.hpp>
#include <skeleton.hpp>
#include <animation.hpp>


class glwidget;

namespace cpe
{
  class debug_mesh;
}

class scene
{
  public:
    scene();

    /** \brief Method called only once at the beginning (load off files ...) */
    void load_model();
    /** \brief Method called at every frame */
    void draw_scene();

    //set glwidget pointer
    void set_glwidget(glwidget *p_widget);

    //set debug_mesh pointer
    void set_debug_mesh(cpe::debug_mesh *p_debug_mesh);

    // (des)activate the drawing of the bind pose
    void set_show_bind_pose(bool);
    // (des)activate the drawing of the skeleton
    void set_show_skeleton(bool);
    // (des)activate the drawing of the skinned cat
    void set_show_skinned(bool);

    void set_speed(int);
    void create_skeleton(float t, int frame1, int frame2);
    void apply_inv_bind_pose();
  private:

    // ground mesh data
    cpe::mesh mesh_0;
    unsigned int texture_id_0;

    // Cat data
    cpe::mesh                     mesh_cat;
    unsigned int                  texture_cat_id;
    cpe::skeleton                *cat_skeleton;
    cpe::animation               *cat_animations;
    std::vector<cpe::v3>          bone_positions;
    std::vector<cpe::quaternion>  bone_orientations;

    glwidget *p_widget;
    cpe::debug_mesh *p_debug_mesh;
    // settings
    bool show_bind_pose;
    bool show_skeleton;
    bool show_skinned;
    float speed;
};

#endif
